/*
 * general header file for ypp-commodities
 */
/*
 *  This is part of ypp-sc-tools, a set of third-party tools for assisting
 *  players of Yohoho Puzzle Pirates.
 * 
 *  Copyright (C) 2009 Ian Jackson <ijackson@chiark.greenend.org.uk>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Yohoho and Puzzle Pirates are probably trademarks of Three Rings and
 *  are used without permission.  This program is not endorsed or
 *  sponsored by Three Rings.
 */

#ifndef CONVERT_H
#define CONVERT_H

#include "common.h"
#include "ocr.h"

#include <pam.h>
#include <time.h>
#include <limits.h>

#include <sys/time.h>


/*----- from rgbimage.c -----*/

#define MAXIMGIDENT 100

typedef struct RgbImage {
  int w, h;
  unsigned char data[];
  /* red   = data[ y*w*3 + x*3 + 0 ] = RI_PIXEL(ri,x,y)[0]
   * green = data[ y*w*3 + x*3 + 1 ] = RI_PIXEL(ri,x,y)[1]
   * blue  = data[ y*w*3 + x*3 + 2 ] = RI_PIXEL(ri,x,y)[2]
   */
} RgbImage;

void identify_rgbimage(const RgbImage *base, Rect portion,
		       char result[MAXIMGIDENT], const char *what);
RgbImage *alloc_rgb_image(int w, int h);

#define RI_PIXEL(ri,x,y) ((ri)->data + ((y)*(ri)->w + (x)) * 3)

/*----- from structure.c -----*/

void find_structure(const CanonImage *im, int *max_relevant_y_r);

void find_islandname(void);
void check_correct_commodities(void);
void read_screenshots(void);
void read_one_screenshot(void);
void analyse(FILE *tsv_output);

/*----- from convert.c -----*/

extern FILE *screenshot_file;
extern void fetch_with_rsync(const char *stem);

void vwarning(const char *fmt, va_list) FMT(1,0);
void warning(const char *fmt, ...)      FMT(1,2);

void vprogress(const char *fmt, va_list) FMT(1,0);
void progress(const char *fmt, ...)      FMT(1,2);

void vprogress_log(const char *fmt, va_list) FMT(1,0);
void progress_log(const char *fmt, ...)      FMT(1,2);

void vprogress_spinner(const char *fmt, va_list) FMT(1,0);
void progress_spinner(const char *fmt, ...)      FMT(1,2);

enum flags {
  ff_editcharset=     00001,
  ff_singlepage=      00002,
  ff_testservers=     00004,
  
  ff_dict_fetch=      00010,
  ff_dict_submit=     00020,
  ff_dict_pirate=     00040,
  ffs_dict=           00070,

  ff_needisland=      00100,
  ff_printisland=     00200,
  ff_upload=          00400,
  ffs_printisland=    00300,
  ffs_upload=         00500,
};
extern enum flags o_flags;

extern const char *o_ocean, *o_pirate;
extern int o_quiet;

/*----- from pages.c -----*/

void screenshot_startup(void);
void set_yppclient_window(unsigned long wul);
void find_yppclient_window(void);
void take_screenshots(void);
void take_one_screenshot(void);

#define MAX_PAGES 100
extern CanonImage *page_images[MAX_PAGES];
extern int npages;
RgbImage *page0_rgbimage;

extern const char *ocean, *pirate;
extern char *archipelago, *island;


#endif /*CONVERT_H*/
