/*
 * Handling of colour (RGB) images
 */
/*
 *  This is part of ypp-sc-tools, a set of third-party tools for assisting
 *  players of Yohoho Puzzle Pirates.
 * 
 *  Copyright (C) 2009 Ian Jackson <ijackson@chiark.greenend.org.uk>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Yohoho and Puzzle Pirates are probably trademarks of Three Rings and
 *  are used without permission.  This program is not endorsed or
 *  sponsored by Three Rings.
 */

#include "convert.h"

/*
 *
 *
 * widgets.txt
 * format
 *
 *  <magic>
 *
 *  <context>
 *  <string>
 *  ppmnoraw depth 8
 *
 *  <context>
 *  <string>
 *  ppmnoraw depth 8
 *  .
 *
 */

#include "convert.h"

static int identify1(const RgbImage *base, Rect portion,
		    char result[MAXIMGIDENT], const char *what,
		    const char *which) {
  char *dbfile_name= masprintf("#%s-pixmap#.txt",which);
  if (!dbfile_open(dbfile_name))
    goto not_found;

#define FGETSLINE (dbfile_getsline(result,MAXIMGIDENT,__FILE__,__LINE__))

  FGETSLINE;
  dbassert(!strcmp(result,"# ypp-sc-tools pctb pixmaps v1"));
  
  void *row= 0;
  size_t rowa= 0;
  
  for (;;) {
    FGETSLINE;
    if (!result[0] || result[0]=='#') continue;
    if (!strcmp(result,".")) break;

    char magic[10];
    dbfile_getsline(magic,sizeof(magic),__FILE__,__LINE__);

    dbassert(!strcmp(magic,"P3"));
    int w,h,maxval;
    dbassert( dbfile_scanf("%d %d %d",&w,&h,&maxval) == 3);
    dbassert(w>0); dbassert(h>0); dbassert(maxval==255);

    if (rowa < w) {
      rowa= w;
      row= mrealloc(row, rowa*3);
    }
    int diff= w != RECT_W(portion) || h != RECT_H(portion);

    int x,y,i;
    for (y=0; y<h; y++) {
      for (x=0; x<w; x++) {
	for (i=0; i<3; i++) {
	  int c;
	  dbassert( dbfile_scanf("%d",&c) == 1);
	  dbassert(c>=0 && c<=255);
	  diff |= (c != RI_PIXEL(base, portion.tl.x + x, portion.tl.y + y)[i]);
	}
      }
    }
    if (!diff) {
      progress_log("Identified %s image: %s.",what,result);
      goto found;
    }
  }
 not_found:
  result[0]= 0;

 found:
  dbfile_close();
  free(dbfile_name);
  return !!result[0];
}

static int identify(const RgbImage *base, Rect portion,
		    char result[MAXIMGIDENT], const char *what) {
  return identify1(base,portion,result,what, "master") ||
         identify1(base,portion,result,what, "local");
}

static void fwrite_ppm(FILE *f, const RgbImage *base, Rect portion) {
  int x,y,i;
  fprintf(f,"P3\n%d %d\n255\n", RECT_W(portion), RECT_H(portion));
  for (y=portion.tl.y; y<=portion.br.y; y++) {
    for (x=portion.tl.x; x<=portion.br.x; x++) {
      putc(' ',f);
      for (i=0; i<3; i++)
	fprintf(f," %3d", RI_PIXEL(base,x,y)[i]);
    }
    putc('\n',f);
  }
  sysassert(!ferror(f));
  sysassert(!fflush(f));
}

void identify_rgbimage(const RgbImage *base, Rect portion,
		       char result[MAXIMGIDENT], const char *what) {
  static int synced;

  if (!synced) {
    fetch_with_rsync("pixmap");
    synced++;
  }
  
  for (;;) {
    int ok= identify(base, portion, result, what);
    if (ok) return;
    
    if (DEBUGP(pixmap))
      fwrite_ppm(stderr,base,portion);

    FILE *resolver= resolve_start();
    if (!resolver)
	fatal("Image recognition failed - unrecognised island.\n"
	      "See FIXME.FIXME\n");

    fprintf(resolver, "pixmap\n" "%s\n", what);
    fwrite_ppm(resolver, base, portion);
    putc('\n',resolver);

    resolve_finish();
  }
}

RgbImage *alloc_rgb_image(int w, int h) {
  RgbImage *ri;
  ri= mmalloc(sizeof(*ri) + w*h*3);
  ri->w= w;
  ri->h= h;
  return ri;
}
